import { app } from '@/app.ts';
import { User } from '@/models/user.ts';
import { CustomError } from '@kevisual/router';
import { checkUsername } from './admin/user.ts';
import { nanoid } from 'nanoid';

app
  .route({
    path: 'user',
    key: 'list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const users = await User.findAll({
      attributes: ['id', 'username', 'description', 'needChangePassword'],
      order: [['updatedAt', 'DESC']],
      logging: false,
    });
    ctx.body = users;
  })
  .addTo(app);

app
  .route({
    path: 'user',
    key: 'update',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id, username, password, description } = ctx.query.data || {};
    if (!id) {
      throw new CustomError(400, 'id is required');
    }
    const user = await User.findByPk(id);
    if (user.id !== tokenUser.id) {
      throw new CustomError(403, 'Permission denied');
    }

    if (!user) {
      throw new CustomError(500, 'user not found');
    }
    if (username) {
      user.username = username;
    }
    if (password) {
      user.createPassword(password);
    }
    if (description) {
      user.description = description;
    }
    await user.save();
    ctx.body = {
      id: user.id,
      username: user.username,
      description: user.description,
      needChangePassword: user.needChangePassword,
    };
  })
  .addTo(app);

app
  .route({
    path: 'user',
    key: 'add',
    middleware: ['auth-admin'],
  })
  .define(async (ctx) => {
    const { username, password, description } = ctx.query.data || {};
    if (!username) {
      throw new CustomError(400, 'username is required');
    }
    checkUsername(username);
    const findUserByUsername = await User.findOne({ where: { username } });
    if (findUserByUsername) {
      throw new CustomError(400, 'username already exists');
    }
    const pwd = password || nanoid(6);
    const user = await User.createUser(username, pwd, description);
    ctx.body = {
      id: user.id,
      username: user.username,
      description: user.description,
      needChangePassword: user.needChangePassword,
      password: pwd,
    };
  })
  .addTo(app);
